var _a;
import { RawAccount } from "cojson";
import { Account, CoValueBase, Group, InitValues, ItemsSym, Ref, SchemaInit, co, inspect, isRefEncoded, makeRefs, } from "../internal.js";
import { encodeSync, decodeSync } from "@effect/schema/Schema";
/** @category CoValues */
// eslint-disable-next-line @typescript-eslint/no-explicit-any
export class CoList extends Array {
    static Of(item) {
        var _b, _c;
        // TODO: cache superclass for item class
        return _c = class CoListOf extends _a {
                constructor() {
                    super(...arguments);
                    this[_b] = item;
                }
            },
            _b = co.items,
            _c;
    }
    /** @deprecated Use UPPERCASE `CoList.Of` instead! */
    static of(..._args) {
        throw new Error("Can't use Array.of with CoLists");
    }
    get _schema() {
        return this.constructor._schema;
    }
    get _owner() {
        return this._raw.group instanceof RawAccount
            ? Account.fromRaw(this._raw.group)
            : Group.fromRaw(this._raw.group);
    }
    /** @category Content */
    get _refs() {
        return makeRefs((idx) => this._raw.get(idx), () => Array.from({ length: this._raw.entries().length }, (_, idx) => idx), this._loadedAs, (_idx) => this._schema[ItemsSym]);
    }
    get _edits() {
        throw new Error("Not implemented");
    }
    get _loadedAs() {
        return Account.fromNode(this._raw.core.node);
    }
    static get [Symbol.species]() {
        return Array;
    }
    constructor(options) {
        super();
        Object.defineProperty(this, "_instanceID", {
            value: `instance-${Math.random().toString(36).slice(2)}`,
            enumerable: false,
        });
        if ("owner" in options) {
            this[InitValues] = {
                init: options.init,
                owner: options.owner,
            };
        }
        else if ("fromRaw" in options) {
            Object.defineProperties(this, {
                id: {
                    value: options.fromRaw.id,
                    enumerable: false,
                },
                _raw: { value: options.fromRaw, enumerable: false },
            });
        }
        return new Proxy(this, CoListProxyHandler);
    }
    static create(items, options) {
        return new this({ init: items, owner: options.owner });
    }
    push(...items) {
        for (const item of toRawItems(items, this._schema[ItemsSym])) {
            this._raw.append(item);
        }
        return this._raw.entries().length;
    }
    unshift(...items) {
        for (const item of toRawItems(items, this._schema[ItemsSym])) {
            this._raw.prepend(item);
        }
        return this._raw.entries().length;
    }
    pop() {
        const last = this[this.length - 1];
        this._raw.delete(this.length - 1);
        return last;
    }
    shift() {
        const first = this[0];
        this._raw.delete(0);
        return first;
    }
    splice(start, deleteCount, ...items) {
        const deleted = this.slice(start, start + deleteCount);
        for (let idxToDelete = start + deleteCount - 1; idxToDelete >= start; idxToDelete--) {
            this._raw.delete(idxToDelete);
        }
        let appendAfter = Math.max(start - 1, 0);
        for (const item of toRawItems(items, this._schema[ItemsSym])) {
            console.log(this._raw.asArray(), appendAfter);
            this._raw.append(item, appendAfter);
            appendAfter++;
        }
        return deleted;
    }
    toJSON() {
        const itemDescriptor = this._schema[ItemsSym];
        if (itemDescriptor === "json") {
            return this._raw.asArray();
        }
        else if ("encoded" in itemDescriptor) {
            return this._raw
                .asArray()
                .map((e) => encodeSync(itemDescriptor.encoded)(e));
        }
        else if (isRefEncoded(itemDescriptor)) {
            return this.map((item) => item?.toJSON());
        }
        else {
            return [];
        }
    }
    [inspect]() {
        return this.toJSON();
    }
    static fromRaw(raw) {
        return new this({ fromRaw: raw });
    }
    static schema(def) {
        this._schema || (this._schema = {});
        Object.assign(this._schema, def);
    }
}
_a = CoList;
(() => {
    _a.prototype._type = "CoList";
})();
CoList.load = CoValueBase.load;
CoList.loadEf = CoValueBase.loadEf;
CoList.subscribe = CoValueBase.subscribe;
CoList.subscribeEf = CoValueBase.subscribeEf;
function toRawItems(items, itemDescriptor) {
    const rawItems = itemDescriptor === "json"
        ? items
        : "encoded" in itemDescriptor
            ? items?.map((e) => encodeSync(itemDescriptor.encoded)(e))
            : isRefEncoded(itemDescriptor)
                ? items?.map((v) => v.id)
                : (() => {
                    throw new Error("Invalid element descriptor");
                })();
    return rawItems;
}
function init(list) {
    if (list[InitValues]) {
        const { init, owner } = list[InitValues];
        const raw = owner._raw.createList(toRawItems(init, list._schema[ItemsSym]));
        Object.defineProperties(list, {
            id: {
                value: raw.id,
                enumerable: false,
            },
            _raw: { value: raw, enumerable: false },
        });
        delete list[InitValues];
    }
}
const CoListProxyHandler = {
    get(target, key, receiver) {
        if (typeof key === "string" && !isNaN(+key)) {
            const itemDescriptor = target._schema[ItemsSym];
            const rawValue = target._raw.get(Number(key));
            if (itemDescriptor === "json") {
                return rawValue;
            }
            else if ("encoded" in itemDescriptor) {
                return rawValue === undefined
                    ? undefined
                    : decodeSync(itemDescriptor.encoded)(rawValue);
            }
            else if (isRefEncoded(itemDescriptor)) {
                return rawValue === undefined
                    ? undefined
                    : new Ref(rawValue, target._loadedAs, itemDescriptor).accessFrom(receiver, Number(key));
            }
        }
        else if (key === "length") {
            return target._raw.entries().length;
        }
        else {
            return Reflect.get(target, key, receiver);
        }
    },
    set(target, key, value, receiver) {
        var _b;
        if (key === ItemsSym &&
            typeof value === "object" &&
            SchemaInit in value) {
            (_b = target.constructor)._schema || (_b._schema = {});
            target.constructor._schema[ItemsSym] =
                value[SchemaInit];
            init(target);
            return true;
        }
        if (typeof key === "string" && !isNaN(+key)) {
            const itemDescriptor = target._schema[ItemsSym];
            let rawValue;
            if (itemDescriptor === "json") {
                rawValue = value;
            }
            else if ("encoded" in itemDescriptor) {
                rawValue = encodeSync(itemDescriptor.encoded)(value);
            }
            else if (isRefEncoded(itemDescriptor)) {
                rawValue = value.id;
            }
            target._raw.replace(Number(key), rawValue);
            return true;
        }
        else {
            return Reflect.set(target, key, value, receiver);
        }
    },
    defineProperty(target, key, descriptor) {
        var _b;
        if (descriptor.value &&
            key === ItemsSym &&
            typeof descriptor.value === "object" &&
            SchemaInit in descriptor.value) {
            (_b = target.constructor)._schema || (_b._schema = {});
            target.constructor._schema[ItemsSym] =
                descriptor.value[SchemaInit];
            init(target);
            return true;
        }
        else {
            return Reflect.defineProperty(target, key, descriptor);
        }
    },
    has(target, key) {
        if (typeof key === "string" && !isNaN(+key)) {
            return Number(key) < target._raw.entries().length;
        }
        else {
            return Reflect.has(target, key);
        }
    },
};
//# sourceMappingURL=coList.js.map