// eslint-disable-next-line @typescript-eslint/no-explicit-any
export function fulfillsDepth(depth, value) {
    if (value._type === "CoMap" ||
        value._type === "Group" ||
        value._type === "Account") {
        if (Array.isArray(depth) && depth.length === 1) {
            return Object.entries(value).every(([key, item]) => {
                return value._refs[key]
                    ? item && fulfillsDepth(depth[0], item)
                    : true;
            });
        }
        else {
            for (const key of Object.keys(depth)) {
                const map = value;
                if (!map._refs[key]) {
                    continue;
                }
                if (!map[key]) {
                    return false;
                }
                if (!fulfillsDepth(depth[key], map[key])) {
                    return false;
                }
            }
            return true;
        }
    }
    else if (value._type === "CoList") {
        if (depth.length === 0) {
            return true;
        }
        else {
            const itemDepth = depth[0];
            return value.every((item, i) => value._refs[i]
                ? item && fulfillsDepth(itemDepth, item)
                : true);
        }
    }
    else if (value._type === "CoStream") {
        if (depth.length === 0) {
            return true;
        }
        else {
            const itemDepth = depth[0];
            return Object.values(value.perSession).every((entry) => entry.value && fulfillsDepth(itemDepth, entry.value));
        }
    }
    else if (value._type === "BinaryCoStream") {
        return true;
    }
    else {
        console.error(value);
        throw new Error("Unexpected value type: " + value._type);
    }
}
//# sourceMappingURL=deepLoading.js.map